Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
oplot.ml1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48Debug.print "* Loading oplot";; (* Simple mathematical plotter library for `ocaml` with fast graphics (opengl), LaTeX display, and high quality vector output (xfig, postscript or PDF) Copyright (c) 2006-2025 VU NGOC San *) (* This program was initially developped by the author at the University of Grenoble a long time ago. *) (* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. *) include Oplot_intf module Points = Points module type GRAPHICS = Make_graphics.GRAPHICS module Core = Make_core.Make (Make_graphics.Dummy) (* The initial modules are grouped and re-split into two main modules: Plt for all usual functions and Internal for specific needs like goplot. TODO split this from stratch. *) (* module type PltS = Make_plt.S *) module Plt : S = struct include Core module Internal = Core end module Make (G : GRAPHICS) : S = struct module Main = Make_core.Make (G) include Main module Internal = Main end (* Local Variables: compile-command:"cd ..;dune build" End: *)