Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
oplot.ml1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45Debug.print "* Loading oplot" (* Simple mathematical plotter library for `ocaml` with fast graphics (opengl), LaTeX display, and high quality vector output (xfig, postscript or PDF) Copyright (c) 2006-2022 VU NGOC San *) (* This program was initially developped by the author at the University of Grenoble a long time ago. *) (* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. *) module Points = Points module Common = Common module type GRAPHICS = Make_graphics.GRAPHICS module Main = Make.Make (Make_graphics.Dummy) (* The initial modules are grouped and re-split into two main modules: Plt for all usual functions and Internal for specific needs like goplot. TODO split this from stracth. *) (* module type PltS = Make_plt.S *) module Plt = struct include Common include Main module Internal = Main end module Make = Make.Make (* Local Variables: compile-command:"cd ..;dune build" End: *)